require "bit"

local function load(filename)
	local tilemap = Sprite.new()
-- Bits on the far end of the 32-bit global tile ID are used for tile flags (flip, rotate)
local FLIPPED_HORIZONTALLY_FLAG = 0x80000000;
local FLIPPED_VERTICALLY_FLAG = 0x40000000;
local FLIPPED_DIAGONALLY_FLAG = 0x20000000;

	local map = loadfile(filename)()

	for i=1, #map.tilesets do
		local tileset = map.tilesets[i]
		
		tileset.sizex = math.floor((tileset.imagewidth - tileset.margin + tileset.spacing) / (tileset.tilewidth + tileset.spacing))
		tileset.sizey = math.floor((tileset.imageheight - tileset.margin + tileset.spacing) / (tileset.tileheight + tileset.spacing))
		tileset.lastgid = tileset.firstgid + (tileset.sizex * tileset.sizey) - 1

		tileset.texture = Texture.new(tileset.image)
	end

	local function gid2tileset(map, gid)
		for i=1, #map.tilesets do
			local tileset = map.tilesets[i]
		
			if tileset.firstgid <= gid and gid <= tileset.lastgid then
				return tileset
			end
		end
	end	
	
	for i=1, #map.layers do
    
    local tileset = map.tilesets[i]
    
		local layer = map.layers[i]

		local tilemaps = {}
		local group = Sprite.new()

		for y=1,layer.height do
			for x=1,layer.width do
        -- Variables to let us know if the tile is flipped or rotated
				local flipHor, flipVer, flipDia = 0, 0, 0
        
				local i = x + (y - 1) * layer.width
				local gid = layer.data[i]
        -- If not empty tile
			local flipHor
			local flipVer
			local flipDia
				if gid ~= 0 then
					-- Read flipping flags
					flipHor = bit.band(gid, FLIPPED_HORIZONTALLY_FLAG)
					flipVer = bit.band(gid, FLIPPED_VERTICALLY_FLAG)
					flipDia = bit.band(gid, FLIPPED_DIAGONALLY_FLAG)
					-- Convert flags to gideros style
					if(flipHor ~= 0) then flipHor = 4 end --TileMap.FLIP_HORIZONTAL end
					if(flipVer ~= 0) then flipVer = 2 end --TileMap.FLIP_VERTICAL end
					if(flipDia ~= 0) then flipDia = 1 end --TileMap.FLIP_DIAGONAL end
					-- Clear the flags from gid so other information is healthy
					gid = bit.band(gid, bit.bnot(bit.bor(FLIPPED_HORIZONTALLY_FLAG, FLIPPED_VERTICALLY_FLAG, FLIPPED_DIAGONALLY_FLAG)))
				end
        
				local tileset = gid2tileset(map, gid)
				
				if tileset then
					local tilemap = nil
					if tilemaps[tileset] then
						tilemap = tilemaps[tileset]
					else
						tilemap = IsometricTileMap.new(layer.width, 
													   layer.height,
													   tileset.texture,
													   tileset.tilewidth,
													   tileset.tileheight,
													   tileset.spacing,
													   tileset.spacing,
													   tileset.margin,
													   tileset.margin,
													   map.tilewidth,
													   map.tileheight)
						tilemaps[tileset] = tilemap
						group:addChild(tilemap)
					end
					
					local tx = (gid - tileset.firstgid) % tileset.sizex + 1
					local ty = math.floor((gid - tileset.firstgid) / tileset.sizex) + 1
					
          -- Set the tile with flip info
					tilemap:setTile(x, y, tx, ty, bit.bor(flipHor, flipVer, flipDia))
					-- Reset vars, so they dont confuse us in the next iteration
					flipHor, flipVer, flipDia = 0, 0, 0
          
				end
			end
		end
		group:setX(400)
		group:setAlpha(layer.opacity)
		
		tilemap:addChild(group)
	end
	
	return tilemap
end

local tilemap = load("iso-test-vertexz.lua")
--stage:addChild(tilemap)





-- Load the mask images
local mask = Bitmap.new(Texture.new("light8.png"))

-- Set its anchor point in the center
mask:setAnchorPoint(0.5, 0.5)

-- Scale it up by a factor of 3
mask:setScale(2)

-- Position the mask in the middle of the simulator
mask:setPosition(application:getLogicalHeight()/2,application:getLogicalWidth()/2)

-- Set its speed and direction values
mask.speedX = 3
mask.speedY = 3
mask.dirX=1
mask.dirY=1




-- Position the tilemap in the middle of the simulator
--tilemap:setPosition(application:getLogicalHeight()/2,application:getLogicalWidth()/2)

--Now set the blend mode
--tilemap:setBlendMode(Sprite.MULTIPLY)

local frame = Sprite.new()

frame:addChild(mask)

--frame:addChild(tilemap)

stage:addChild(frame)



--render target
local tilemapx, tilemapy = tilemap:getPosition()
tilemap:setX(tilemapx*2.5)
local rt = RenderTarget.new( tilemap:getWidth(), tilemap:getHeight())

--bitmap with rendertarget as texture
local scene = Bitmap.new(rt)
local rendered = Sprite.new()
rendered:addChild(scene)
rendered:setBlendMode("multiply")
frame:addChild(rendered)

rt:draw(tilemap)

--stage:addChild(scene)

--rendertarget

-- Set the background color of the stage to black
stage:setBackgroundColor(0, 0, 0.0)

-- Define the onEnterFrame event function. This deals with the positioning of the mask
function onEnterFrame(event)
	--rt:clear(0x000000,1)
	--rt:draw(frame)

--[[
	-- Get the position of the maks
	local x,y = mask:getPosition()
	-- Calculate the new position
	x = x + (mask.speedX * mask.dirX)
	y = y + (mask.speedY * mask.dirY)
	-- Check if the mask reaches the edges of the canvas and then mirror its movement.
	if x < 0 then
		mask.dirX = 1
	end
		
	if x > application:getLogicalHeight() then
		mask.dirX = -1
	end

	if y < 0 then
		mask.dirY = 1
	end
		
	if y > application:getLogicalWidth() then
		mask.dirY = -1
	end
	-- Set the new position of the mask
	mask:setPosition(x,y)
	]]
end


local dragging, startx, starty

local function onMouseDown(event)
	dragging = true
	startx = event.x
	starty = event.y
	local xp,yp = frame:globalToLocal(event.x,event.y)
	mask:setPosition(xp,yp)
end

local function onMouseMove(event)
	if dragging then
		local dx = event.x - startx
		local dy = event.y - starty
		frame:setX(frame:getX() + dx)
		frame:setY(frame:getY() + dy)
		startx = event.x
		starty = event.y
	end
	
	
	
end

local function onMouseUp(event)
	dragging = false
end
-- Register an enterFrame event handler function
stage:addEventListener(Event.ENTER_FRAME,onEnterFrame)


stage:addEventListener(Event.MOUSE_DOWN, onMouseDown)
stage:addEventListener(Event.MOUSE_MOVE, onMouseMove)
stage:addEventListener(Event.MOUSE_UP, onMouseUp)
